# Color Channel Image Reconstructor 16/8bit Version
# Ver. 0.01b
# Silent & Verbose Version
# (c)2021 Ohnishi, Yasuo
from ij import IJ, ImagePlus 
from ij import plugin
from ij import WindowManager as WM
from ij.io import DirectoryChooser
from ij.io import OpenDialog
from ij.gui import GenericDialog
from ij.plugin import ChannelSplitter
from ij.plugin import Duplicator
from ij.process import ImageConverter
from loci.plugins import BF
import os
from org.python.core import codecs
codecs.setDefaultEncoding('utf-8')

def convValue(Value, Mode):
#   Converting Luminance Value each in 8, 16, 32bit mode
    if Mode == "16-bit":
        Value = Value * 256
    elif Mode == "32-bit":
        Value = Value * 256 * 256
    else:
        Value = Value
    return Value
#
#####################################################
def RGBmanipulation(folder,filename,parm):
#  input Parameters
    if parm is not None:
 # if parm is not null object, then...
        Mode, BaseCh, MixCh, MixRatio, MixMode, Show, GIMPout, Gamma, MixCh1, MixRatio1, MixMode1, Gamma1 = parm
        print(Mode, BaseCh, MixCh, MixRatio, MixMode, Show, GIMPout, Gamma) 
#     Mode: Image bit depth 16-bit or 8-bit
#     Show: Show working windows or not
#     parm_Binary: Threshold of fore / background
#     parm_Distance: Lightness of Distance adjusting Layer
#     parm_Dark: Threshold of Dark part adjustment image
#     parm_Prph: Threshold of Periphery adjustment image
#     parm_FGTh: Threshold of Foreground mask

# Always Open file with Bio-format Plugin
    imps = BF.openImagePlus(folder + filename) # Open file with Bio-format Plugin
    imp0 = imps[0] #imp0: Original Image 
    if Mode == "Auto":
        bitdepth = imp0.getBitDepth()
        if bitdepth == 8:
            Mode = "8-bit"
        elif bitdepth ==16:
            Mode = "16-bit"
        elif bitdepth ==32:
            Mode = "32-bit"
        else:
            Mode = "16-bit"
    if Show == "Yes":
 # Windows Showing Mode
        imp0.show()
#------
    impsCh = ChannelSplitter.split(imp0) #Channel Splitting of imp0
    for i in impsCh: # it needs to convert Grayscale image
        if Mode == "8-bit":
            ImageConverter(i).convertToGray8()
        elif Mode == "16-bit":
            ImageConverter(i).convertToGray16()
        else:
            ImageConverter(i).convertToGray32()
    impGray = Duplicator().run(imp0)
    if MixCh == "Luminosity" or  MixCh1 == "Luminosity":
        IJ.run(impGray, "Conversions...", "weighted")
    if Mode == "16-bit":
        ImageConverter(impGray).convertToGray16()
    elif Mode == "8-bit":
        ImageConverter(impGray).convertToGray8()
## R ch.
    impR =  impsCh[0]
    IJ.saveAsTiff(impR, folder + filename +"_R.tif")
## G ch.
    impG =  impsCh[1]
    IJ.saveAsTiff(impG, folder + filename +"_G.tif")
## B ch. & B x 0.3
    impB =  impsCh[2]
    IJ.saveAsTiff(impB, folder + filename +"_B.tif")
# Show Windows
    imp0.close()
    if Show == "Yes":
        impR.show()
        impG.show()
        impB.show()
    if BaseCh == "R":
        imp1 = impR
    elif BaseCh == "G":
        imp1 = impG
    elif BaseCh == "B":
        imp1 = impB
# 1st Mixing Ch.
    if MixCh == "R":
        imp2org = impR
        imp2 = Duplicator().run(impR)
    elif MixCh == "G":
        imp2org = impG
        imp2 = Duplicator().run(impG)
    elif MixCh == "B":
        imp2org = impB
        imp2 = Duplicator().run(impB)
    else:
        imp2org = impGray
        imp2 = Duplicator().run(impGray)
# 2nd Mixing Ch.
    if MixCh1 == "R":
        imp3org = impR
        imp3 = Duplicator().run(impR)
    elif MixCh1 == "G":
        imp3org = impG
        imp3 = Duplicator().run(impG)
    elif MixCh1 == "B":
        imp3org = impB
        imp3 = Duplicator().run(impB)
    elif MixCh1 == "Luminosity" or MixCh1 == "Lightness":
        imp3org = impGray
        imp3 = Duplicator().run(impGray)
#        IJ.showMessage("imp3 is " + MixCh1)
    else:
        MixRatio1 = 0.0
##      Convert Base Channel
    Ratio1 = str(1.0 - (MixRatio + MixRatio1))
    IJ.run(imp1, "Multiply...", "value=" + Ratio1)
    if Show == "Yes":
        imp1.show()
##      Convert 1st Mixing Channel
    Ratio2 = str(MixRatio)
    IJ.run(imp2, "Multiply...", "value=" + Ratio2)
    if Show == "Yes":
        imp2.show()
###    Multiply Gammma for 1st Mixing Ch.
    IJ.run(imp2, "Gamma...", "value=" + str(Gamma))
#####  Mix Base Channel and 1st Mixing Channel
    ic = plugin.ImageCalculator()
    imp10 = ic.run("Add create", imp1, imp2) # No window made
    if MixMode == "Darken":
        ic = plugin.ImageCalculator()
        imp11 = ic.run("min create", imp10, imp2org) # No window made
    elif MixMode == "Lighten":
        ic = plugin.ImageCalculator()
        imp11 = ic.run("max create", imp10, imp2org) # No window made
    else:
        imp11 = imp10
    if MixCh1 == "None":
        IJ.saveAsTiff(imp11, folder + filename +"_" + BaseCh + "x" + Ratio1 + "_" + MixCh + "x" + Ratio2 + MixMode + "Gm" + str(Gamma) + ".tif")
        if GIMPout =="Yes":
            IJ.saveAsTiff(imp11, folder + filename +"_Ch_Reconstructed.tif")
    else: 
#        IJ.showMessage("2nd Mixing Channel " + MixCh1)
######### Mixing 2nd Channel #############
##      Convert Mixing Channel
        Ratio3 = str(MixRatio1)
        IJ.run(imp3, "Multiply...", "value=" + Ratio2)
        if Show == "Yes":
           imp3.show()
###    Multiply Gammma for 2nd Mixing Ch.
        IJ.run(imp3, "Gamma...", "value=" + str(Gamma))
#  Mix imp11 and 2nd Mixing Channel
        ic = plugin.ImageCalculator()
        imp20 = ic.run("Add create", imp11, imp3) # No window made
        if MixMode1 == "Darken":
            ic = plugin.ImageCalculator()
            imp21 = ic.run("min create", imp20, imp3org) # No window made
        elif MixMode1 == "Lighten":
            ic = plugin.ImageCalculator()
            imp21 = ic.run("max create", imp20, imp3org) # No window made
        else:
            imp21 = imp20
# output for 2nd Mixed Ch
        IJ.saveAsTiff(imp21, folder + filename +"_" + BaseCh + "x" + Ratio1 + "_" + MixCh + "x" + Ratio2 + MixMode + "Gm" + str(Gamma) +  "_" + MixCh1 + "x" + Ratio3 + MixMode1 + "Gm" + str(Gamma1) + ".tif")
        if GIMPout =="Yes":
            IJ.saveAsTiff(imp21, folder + filename +"_Ch_Reconstructed.tif")
        imp20.hide()
        imp20.close()
        if MixMode1 != "Normal":
            imp21.hide()
            imp21.close()

#    imp10.getProcessor().setLut(None)
    imp10.hide()
    imp10.close()
    if MixMode != "Normal":
        imp11.hide()
        imp11.close()
    if MixCh1 != "None":
        imp3.hide()
        imp3.close()
    imp2.hide()
    impR.hide()
    impG.hide()
    impB.hide()
    impGray.hide()
    imp2.close()
    impR.close()
    impG.close()
    impB.close()
    impGray.close()
#####################################################

# Get parameters from the input DialogBox 
def getParameters():
    gd = GenericDialog("Color Channel Reconstructor / Enter Parameters")
    selectBaseCh = ["R","G","B"]
    gd.addChoice("Channel for Reconstructuon", selectBaseCh, selectBaseCh[1])
#
    gd.addMessage("[1st Mixing Channel]")
    selectMixCh = ["R","G","B","Luminosity", "Lightness"]
    gd.addChoice("Mixing Channel", selectMixCh, selectMixCh[0])
    gd.addNumericField("Channel Mixing Ratio (0 to 1.0)", 0.5, 1)
    selectAdjst = ["Darken","Normal","Lighten"]
    gd.addChoice("Mixing Mode", selectAdjst, selectAdjst[1])
    gd.addNumericField("Gamma for Mixing Ch.(0 to 5.0)", 1.0, 1)
#
    gd.addMessage("[2nd Mixing Channel]")
    selectMixCh1 = ["None","R","G","B","Luminosity", "Lightness"]
    gd.addChoice("Mixing Channel", selectMixCh1, selectMixCh1[0])
    gd.addNumericField("Channel Mixing Ratio (0 to 1.0)", 0.0, 1)
    selectAdjst1 = ["Darken","Normal","Lighten"]
    gd.addChoice("Mixing Mode", selectAdjst1, selectAdjst1[1])
    gd.addNumericField("Gamma for Mixing Ch.(0 to 5.0)", 1.0, 1)
#
    gd.addMessage("    ")
    selectGIMPoutput = ["Yes","No"]
    gd.addChoice("Output File name for GIMP plugin?", selectGIMPoutput, selectGIMPoutput[0])
    showWin = ["Yes","No"]
    gd.addChoice("Show working windows?", showWin, showWin[0])
    gd.showDialog()
    Mode   = str("Auto")
    BaseCh   = str(gd.getNextChoice())
# get 1st Mixing Ch.
    MixCh   = str(gd.getNextChoice())
    MixRatio  = gd.getNextNumber()
    MixMode   = str(gd.getNextChoice())
    Gamma =  gd.getNextNumber()
# get 2nd Mixing Ch.
    MixCh1   = str(gd.getNextChoice())
    MixRatio1  = gd.getNextNumber()
    MixMode1   = str(gd.getNextChoice())
    Gamma1 =  gd.getNextNumber()

    GIMPout   = str(gd.getNextChoice())
    Show   = str(gd.getNextChoice())
    if MixRatio > 1.0:
        MixRatio = 1.0
    elif MixRatio < 0.0:
        MixRatio = 0.0
    if Gamma > 5.0:
        Gamma = 5.0
    elif Gamma < 0.0:
        Gamma = 0.0

    if MixRatio + MixRatio1 > 1.0:
        MixRatio1 = 1.0 - MixRatio
        IJ.showMessage("2nd Mixing Ratio was changed to " + str(MixRatio1))
    elif MixRatio1 < 0.0:
        MixRatio1 = 0.0
    if Gamma1 > 5.0:
        Gamma1 = 5.0
    elif Gamma1 < 0.0:
        Gamma1 = 0.0
    if MixCh == "Luminosity" and MixCh1 == "Lightness":
        IJ.showMessage("2nd Lightness Ch. will be changed to None!")
        MixCh1 = "None"
    elif MixCh1 == "Luminosity" and MixCh == "Lightness":
        IJ.showMessage("2nd Luminosity Ch. will be changed to None!")
        MixCh1 = "None"
    if gd.wasCanceled():  
        Mode   = "Stop"
        IJ.log("Canceled")
    if BaseCh == MixCh:
        IJ.showMessage("Mixing Channel is same with Reconstructing Channel!")
#        Mode   = "Stop"
    return Mode, BaseCh, MixCh, MixRatio, MixMode, Show, GIMPout, Gamma, MixCh1, MixRatio1, MixMode1, Gamma1

# Main
od = OpenDialog("Choose a file")
folder = od.getDirectory()
file = od.getFileName()
path = folder + file
IJ.log(path)
IJ.log(folder)
IJ.log(file)
parm = getParameters()
print ("Parm[0]=", parm[0])
if parm[0] <> "Stop":
    RGBmanipulation(folder,file,parm)
    IJ.log("Finish")
#    IJ.run("Quit")
